#include <math.h>
#include "RKData.h"
#include "Component.h"
#include "Cell.h"

CCell::CCell()
{
	neqs = 1;	// one equation for voltage;
// cell shape
	Dm = 60.0;  // micrometer, diameter of the cell
	Lm = 70.0;  // micrometer, length of the cell
	Am = 3.14 * Dm * Lm; // squre_micrometer, surface area
	Cm = 0.01 * Am;
	InvCm = 1/Cm;
	fac = 10;  // conversion factor (mS/cm^2) -> (pS/micrometer^2)
// reversal potentials
	Eleak = -65.0;   // mV
	ENa = 50.0;      // mV
	EK = -100.0;     // mV
	ECa = 50.0;      // mV
// channel density
	dNa = 0.05;      // mS/cm^2
	dK  = 0.15;
	dTCa = 0.03;
	dGleak = 0.002; 
	Gleak = Am * dGleak * fac;
// injected current
	Iinj = 2000;     // pA
	Ton  = 20;       // ms
	Toff = 220;      // ms 
}

CCell::~CCell()
{
	delete pNaChannel;
	delete pKChannel;
	delete pTCaChannel;
};

int CCell::initCell(double h)
{
	pNaChannel  = new CNaChannel();
	pKChannel   = new CKChannel();
	pTCaChannel = new CTCaChannel();

	pNaChannel->G = dNa * Am * fac;
	pKChannel->G = dK * Am * fac;
	pTCaChannel->G = dTCa * Am * fac;

	neqs = addComponent(pNaChannel);
	neqs = addComponent(pKChannel);
	neqs = addComponent(pTCaChannel);

	initState(neqs,h); // allocate the working spaces

	y[0]=Eleak;
	pNaChannel->initState(y);
	pKChannel->initState(y);
	pTCaChannel->initState(y);

	return neqs;
}

int CCell::addComponent(CComponent* pComp)
{
	pComp->bp = neqs;
	neqs += pComp->neqs;
	return neqs;
}

void CCell::rhs(double x, double*y, double* dy)
{
	double 	INa, IK, ITCa;
	double v = y[0];
	double c = (x<Ton || x>Toff)? 0 : Iinj;
	int bx;

	bx = pNaChannel->bp;
	INa = pNaChannel->G * pow(y[bx],3.0) * y[bx+1]  * (v-ENa);
	
	bx = pKChannel->bp;
	IK = pKChannel->G * pow(y[bx+0],4.0) * (v-EK);

	bx = pTCaChannel->bp;
	ITCa = pTCaChannel->G * pow(y[bx+0],2.0) * y[bx+1] * (v - ECa);

	dy[0] = InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa );

	pNaChannel->rhs(x,y,dy);
	pKChannel->rhs(x,y,dy);
	pTCaChannel->rhs(x,y,dy);
}
